/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 2.1.2008
 *
 * This is the Application entry point. Its the main class for the Application (or Game).
 */
module application;

import singleton;
import task;
import actionlistener;
import renderer;
import fontwriter;
alias fontwriter.FontWriter.getInstance FontWriter;
import appstate;
import gamestate;
import loader;
import game;
import menue;
import config;
import timer;

import input;

import std.stdio;
import std.stream;

class Application : Task, ActionListener {
public:
    mixin Singleton;
    override void init() {
        Loader.display();
        FontWriter.init();
        myState = APP_STATE.MENUE_RUNNING;
        myMenue = new Menue();
        myMenue.init(APP_STATE.STARTIMPL_MAIN);
        myMenue.addActionListener(this);
    };
    override int refresh() {
        foreach(key; Input.getInstance().keys)
            if (key.sdl_key == 27) {
                myState = APP_STATE.QUIT;
            };

        switch(myState) {
            case APP_STATE.GAME_RUNNING:
                switch (myGame.refresh()) {
                    case GAME_STATE.RUNNING:
                        break;
                    case GAME_STATE.LOST:
                        myGame.uninit();
                        delete myGame;
                        myMenue = new Menue();
                        myMenue.init(APP_STATE.STARTIMPL_LOST);
                        myMenue.addActionListener(this);
                        myState = APP_STATE.MENUE_RUNNING;
                        break;
                    case GAME_STATE.WON_BRONZE:
                        myGame.uninit();
                        delete myGame;
                        addPrize(1);
                        myMenue = new Menue();
                        myMenue.init(APP_STATE.STARTIMPL_WON_BRONZE);
                        myMenue.addActionListener(this);
                        myState = APP_STATE.MENUE_RUNNING;
                        break;
                    case GAME_STATE.WON_SILVER:
                        myGame.uninit();
                        delete myGame;
                        addPrize(2);
                        myMenue = new Menue();
                        myMenue.init(APP_STATE.STARTIMPL_WON_SILVER);
                        myMenue.addActionListener(this);
                        myState = APP_STATE.MENUE_RUNNING;
                        break;
                    case GAME_STATE.WON_GOLD:
                        myGame.uninit();
                        delete myGame;
                        addPrize(3);
                        myMenue = new Menue();
                        myMenue.init(APP_STATE.STARTIMPL_WON_GOLD);
                        myMenue.addActionListener(this);
                        myState = APP_STATE.MENUE_RUNNING;
                        break;
                };
                break;
            case APP_STATE.MENUE_RUNNING:
                myMenue.refresh();
                break;
            case APP_STATE.START_GAME:
                myMenue.uninit();
                delete myMenue;
                myGame = new Game();
                myGame.init(attribute);
                myState = APP_STATE.GAME_RUNNING;
                break;
            case APP_STATE.QUIT:
                return 0;
            default: break;
        };

        Renderer.getInstance().join2D();
        FontWriter.Size(1);
        FontWriter.Bold(1);
        FontWriter.write2D("DEngine :: beta 1",0,Config["window_height"]-5);
        FontWriter.Bold(0);
        FontWriter.write2D("Frames per Second: " ~ std.string.toString( Timer.getInstance().fps ), 10, Config["window_height"]-20);
        FontWriter.write2D("Ticks: " ~ std.string.toString( Timer.getInstance().ticks ), 10, Config["window_height"]-35);

        return 1;

    };
    override void uninit() {
        if (myMenue) {
            myMenue.uninit();
            delete myMenue;
        };
        if (myGame) {
            myGame.uninit();
            delete myGame;
        };
        FontWriter.uninit();
    };
    void actionPerformed(char[] theCommand, char[] theAttribute) {
        switch(theCommand) {
            case "Menue":
                switch(theAttribute) {
                    case "ShutDown":
                        myState = APP_STATE.QUIT;
                        break;
                };
                break;
            case "Game":
                myState = APP_STATE.START_GAME;
                attribute = theAttribute;
                break;
            default: break;
        };
    };

private:
    void addPrize(int p) {
        File savegame = new File();
        savegame.open("resources/savegame", FileMode.In);
        int[]   savegameData;
        while (!savegame.eof) {
            savegameData ~= 0;
            savegame.readf(&savegameData[length-1]);
        };
        savegame.close();
        savegame.open("resources/savegame", FileMode.Out);
        if (p>savegameData[0] && attribute == "resources/lvls/lvl1.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[0], " ");
        if (p>savegameData[1] && attribute == "resources/lvls/lvl2.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[1], " ");
        if (p>savegameData[2] && attribute == "resources/lvls/lvl3.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[2], " ");
		if (p>savegameData[3] && attribute == "resources/lvls/lvlSpecial.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[3], " ");
        if (p>savegameData[4] && attribute == "resources/lvls/lvl1b.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[4], " ");
        if (p>savegameData[5] && attribute == "resources/lvls/lvl2b.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[5], " ");
        if (p>savegameData[6] && attribute == "resources/lvls/lvl3b.txt") savegame.writef(p, " ");
            else savegame.writef(savegameData[6], " ");
        savegame.close();
    };

    APP_STATE   myState;
    char[]      attribute;
    Menue       myMenue;
    Game        myGame;
};
