/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 10.1.2008
 *
 * A button to click on
 */
module button;

import derelict.opengl.gl;
import derelict.sdl.mixer;

import menueitem;
import point;
import input;
import resourcemanager;
import sound;

import fontwriter;
alias  fontwriter.FontWriter.getInstance FontWriter;

import config;

class Button : MenueItem {
    void init(Point2D thePosition, Point2D theDimension, char[] theText, char[] clickSoundFilename, char[] overSoundFilename) {
        super.init(thePosition, theDimension);
        myText      	= theText;
        myClickSound   	= cast(Sound)ResourceManager.getInstance().need((Sound.classinfo), clickSoundFilename);
        myOverSound   	= cast(Sound)ResourceManager.getInstance().need((Sound.classinfo), overSoundFilename);
    };
    override void uninit() {
        super.uninit();
        ResourceManager.getInstance().unNeed(myClickSound);
        ResourceManager.getInstance().unNeed(myOverSound);
    };

    override void refresh() {
        glLoadIdentity();
        glTranslatef(myPosition.x, myPosition.y, 0);
        glBindTexture(GL_TEXTURE_2D, myTexture.texture);
        glEnable(GL_BLEND);
        if (myBB2D.intersects(Point2D(Input.getInstance().mouse_x, Config["window_height"] - Input.getInstance().mouse_y))) {
            glBegin(GL_QUADS);
                glTexCoord2f(0.0f, 0.148438f); glVertex2f(0,             0);
                glTexCoord2f(1.0f, 0.148438f); glVertex2f(myDimension.x, 0);
                glTexCoord2f(1.0f, 0.292969f); glVertex2f(myDimension.x, myDimension.y);
                glTexCoord2f(0.0f, 0.292969f); glVertex2f(0,             myDimension.y);
            glEnd();

            FontWriter.Bold(1);
            FontWriter.write2D(myText,cast(int)(myPosition.x+myDimension.x-120.0f),cast(int)(myPosition.y+27.0f));
            FontWriter.Bold(0);

            if (!wasOver) Mix_PlayChannel(-1, myOverSound.sound, 0);
			wasOver = true;

            if (buttonDown && !Input.getInstance().mouseLeft) {
                Mix_PlayChannel(-1, myClickSound.sound, 0);
                if (myActionListener) myActionListener.actionPerformed(command, attribute);
            };
            if (Input.getInstance().mouseLeft) buttonDown = true;

        } else {
        	wasOver = false;
            glBegin(GL_QUADS);
                glTexCoord2f(0.0f, 0.0f);      glVertex2f(0,             0);
                glTexCoord2f(1.0f, 0.0f);      glVertex2f(myDimension.x, 0);
                glTexCoord2f(1.0f, 0.148438f); glVertex2f(myDimension.x, myDimension.y);
                glTexCoord2f(0.0f, 0.148438f); glVertex2f(0,             myDimension.y);
            glEnd();
            FontWriter.write2D(myText,cast(int)(myPosition.x+myDimension.x-120.0f),cast(int)(myPosition.y+27.0f));
        };
        glDisable(GL_BLEND);
    };

private:
    char[]          myText;
    bool            buttonDown;
    bool			wasOver;
    Sound			myClickSound;
    Sound			myOverSound;
};
