/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 27.12.2007
 *
 * This is FontWriter Modul, which displays text on the screen. It uses a TextureFont.
 */
module FontWriter;

import derelict.opengl.gl;
import singleton;
import unit;
import resourcemanager;
import texturefont;

import point;
import config;



class FontWriter : Unit {
    mixin Singleton;
    override void init() {
        myFonts ~= cast(TextureFont)ResourceManager.getInstance().need(TextureFont.classinfo, "resources/fonts/nfont_12auf256.png");
        myFonts ~= cast(TextureFont)ResourceManager.getInstance().need(TextureFont.classinfo, "resources/fonts/nfont_14auf256.png");
        myFonts ~= cast(TextureFont)ResourceManager.getInstance().need(TextureFont.classinfo, "resources/fonts/nfont_18auf512.png");
        myFonts ~= cast(TextureFont)ResourceManager.getInstance().need(TextureFont.classinfo, "resources/fonts/nfont_24auf512.png");
        use(myFonts[length-1]);
    };
    override void uninit() {
        foreach(font; myFonts) ResourceManager.getInstance().unNeed(font);
        myCurrentFont = null;
    };
    void Size(int theSize) {
        if (theSize < myFonts.length) use(myFonts[theSize]);
    };
    void Bold(int theBold) {
        if (theBold) myBold = 4;
        else myBold = 0;
    };
    void write2D(char[] text, int x, int y) {
        if (!myCurrentFont) throw new Exception(__FILE__ ~ ": Error with write2D: No Font Loaded.");

        int unicode;

        glBindTexture(GL_TEXTURE_2D, myCurrentFont.texture.texture);
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
        glEnable(GL_BLEND);

        glLoadIdentity();
        glTranslatef(x , y, 0);
        glNormal3f(0,0,1);

        for (int i=0; i<text.length; i++)
        {
            unicode = (cast(int)text[i]-33);
            if (unicode == -1) glTranslatef(5,0,0);
            else {
                if (unicode < 0)    unicode = 3;
                if (unicode > 112)  unicode = 3;

                glBegin(GL_QUADS);
                    glTexCoord2f( myCurrentFont.letters[unicode].uv[0+myBold].x, myCurrentFont.letters[unicode].uv[0+myBold].y );
                        glVertex2f( 0, 0);
                    glTexCoord2f( myCurrentFont.letters[unicode].uv[1+myBold].x, myCurrentFont.letters[unicode].uv[1+myBold].y );
                        glVertex2f( myAligner.x*(myCurrentFont.letters[unicode].uv[1+myBold].x-myCurrentFont.letters[unicode].uv[0+myBold].x), 0);
                    glTexCoord2f( myCurrentFont.letters[unicode].uv[2+myBold].x, myCurrentFont.letters[unicode].uv[2+myBold].y );
                        glVertex2f( myAligner.x*(myCurrentFont.letters[unicode].uv[1+myBold].x-myCurrentFont.letters[unicode].uv[0+myBold].x), myAligner.y*(myCurrentFont.letters[unicode].uv[1+myBold].y-myCurrentFont.letters[unicode].uv[2+myBold].y));
                    glTexCoord2f( myCurrentFont.letters[unicode].uv[3+myBold].x, myCurrentFont.letters[unicode].uv[3+myBold].y );
                        glVertex2f( 0, myAligner.y*(myCurrentFont.letters[unicode].uv[0+myBold].y-myCurrentFont.letters[unicode].uv[2+myBold].y));
                glEnd();
                glTranslatef( myAligner.x*(myCurrentFont.letters[unicode].uv[1+myBold].x-myCurrentFont.letters[unicode].uv[0+myBold].x)+1,0,0);
            };
        };
        glDisable(GL_BLEND);
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    };

private:
    void use(TextureFont theFont) {
        myCurrentFont   = theFont;
        myAligner.x     = myCurrentFont.dimension * cast(float)Config["renderer_width"] / cast(float)Config["window_width"];
        myAligner.y     = myCurrentFont.dimension * cast(float)Config["renderer_height"] / cast(float)Config["window_height"];
    };
    Point2D         myAligner;
    TextureFont     myCurrentFont;
    TextureFont[]   myFonts;
    int             myBold;
};
