/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 6.1.2008
 *
 * The Game Module
 */
module game;

import derelict.sdl.sdl;
import derelict.sdl.mixer;
import std.stream;

import scenemanager;
import input;
import player;
import camera;
import renderer;

import resourcemanager;
import heightmap;
import texture2d;
import sound;

import timer;
import gamestate;
import fontwriter;
alias fontwriter.FontWriter.getInstance FontWriter;
import config;

import derelict.opengl.gl;

import std.stdio;
import std.date;

class Game {
public:

    void init(char[] theLevel) {
        myLevelFile = theLevel;

        char[]  heightmapFilename;

        File lvl = new File();
        lvl.open(theLevel, FileMode.In);
        myLevelName = lvl.readLine();
        lvl.readf(&heightmapFilename, &myBackgroundFile);
        lvl.readf(&levelSpeed);
        lvl.close();

        adder = levelSpeed;

        mySceneManager = new SceneManager();
        mySceneManager.init();

        myHeightmap = cast(Heightmap)ResourceManager.getInstance.need(Heightmap.classinfo, heightmapFilename);
        mySceneManager.addItem(myHeightmap);

        myBackgroundTexture = cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, myBackgroundFile);
        mySceneManager.addItem(myBackgroundTexture);

        myExplosionTexture	= cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, "resources/textures/explosion.png");

		myExplosionSound	= cast(Sound)ResourceManager.getInstance.need(Sound.classinfo, "resources/sounds/explosion.wav");
		Mix_PlayChannel(-1, myExplosionSound.sound, 0);

        myPlayer = new Player();
        myPlayer.init();
        myPlayer.position.x = 45.0f;
        myPlayer.position.y = myHeightmap.sizeY / 2.0f;
        myPlayer.position.z = myHeightmap.getHeight(myPlayer.position.x, myPlayer.position.y);
        mySceneManager.addItem(myPlayer);

        myCamera = new Camera();
        myCamera.position.x = -65;
        myCamera.position.y = -33;
        myCamera.position.z = -77;
        mySceneManager.addItem(myCamera);
    };

    GAME_STATE refresh() {


        foreach(key; Input.getInstance().keys) {
            if (key.sdl_key == SDLK_d || key.sdl_key == SDLK_RIGHT) {
                myPlayer.right();
                if (!myPlayer.jumping)myPlayer.position.z = myHeightmap.getHeight(myPlayer.position.x, myPlayer.position.y);
            } else if (key.sdl_key == SDLK_a || key.sdl_key == SDLK_LEFT) {
                myPlayer.left();
                if (!myPlayer.jumping)myPlayer.position.z = myHeightmap.getHeight(myPlayer.position.x, myPlayer.position.y);
            }
            if (key.sdl_key == SDLK_SPACE || key.sdl_key == SDLK_UP || key.sdl_key == SDLK_w) {
                if (!myPlayer.down) myPlayer.jump();
            };
        };

        //myCamera.refresh();
        if (myCamera.position.x > -myHeightmap.sizeX+80) {
        	myCamera.position.x -= adder * Timer.getInstance.ticks * 0.1f;
			adder += Timer.getInstance.ticks * levelSpeed * 0.002f;
        };

        //writef(adder);
        //writefln(myCamera.position.x);

        myPlayer.refresh();

        if (myPlayer.jumping) {
            if (myPlayer.position.z < myHeightmap.getHeight(myPlayer.position.x, myPlayer.position.y)) {
                //writefln("walk");
                myPlayer.walk();
                myPlayer.position.z = myHeightmap.getHeight(myPlayer.position.x, myPlayer.position.y);
            };
        };
        if (myPlayer.position.z <= myHeightmap.sizeZ / 2.0f) {
        	 myState = GAME_STATE.LOST;
        } else if (myPlayer.position.x >= myHeightmap.sizeX/1.01f) {
        	myState = GAME_STATE.LOST;
        } else if (myPlayer.position.x <= -myCamera.position.x-55) {
        	myState = GAME_STATE.LOST;
        };

        if (myState == GAME_STATE.LOST) {
			if (myPlayer.position.x >= myHeightmap.sizeX/2.0f) myState = GAME_STATE.WON_BRONZE;
            if (myPlayer.position.x >= myHeightmap.sizeX/1.333f) myState = GAME_STATE.WON_SILVER;
            if (myPlayer.position.x >= myHeightmap.sizeX/1.01) myState = GAME_STATE.WON_GOLD;
        };

        mySceneManager.refresh();

		Renderer.getInstance.join2D();

		glEnable(GL_BLEND);
		glBindTexture(GL_TEXTURE_2D, myExplosionTexture.texture);
        glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex3f(0,0,0);
			glTexCoord2f(1,0); glVertex3f(25,0, 0);
			glTexCoord2f(1,1); glVertex3f(25, Config["renderer_height"],0);
			glTexCoord2f(0,1); glVertex3f(0, Config["renderer_height"],0);
        glEnd();
        glDisable(GL_BLEND);

        Renderer.getInstance.join3D();

        return myState;
    };

    void uninit() {
        mySceneManager.uninit();
        delete mySceneManager;

        ResourceManager.getInstance.unNeed(myHeightmap);
        ResourceManager.getInstance.unNeed(myBackgroundTexture);
        ResourceManager.getInstance.unNeed(myExplosionTexture);
        ResourceManager.getInstance.unNeed(myExplosionSound);

        myPlayer.uninit();
        delete myPlayer;
    };



private:
    SceneManager    mySceneManager;

    Heightmap       myHeightmap;
    Player          myPlayer;
    Camera          myCamera;
    Texture2D       myBackgroundTexture;
    Texture2D       myExplosionTexture;
    Sound			myExplosionSound;

    float          	levelSpeed;

    char[]          myLevelFile;
    char[]          myLevelName;
    char[]          myBackgroundFile;

    float           adder;

    GAME_STATE      myState = GAME_STATE.RUNNING;
}
