/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 6.1.2008
 *
 * This is the Heightmap Modul, which generates triangles for a outdoor level from a picture.
 */
module heightmap;

import derelict.sdl.sdl;
import derelict.opengl.gl;
import derelict.opengl.extension.arb.multitexture;
import derelict.opengl.extension.arb.shader_objects;

import std.stream;

import resourcemanager;
import resource;
import image;
import texture2d;
import hlslshader;
import vertex;
import point;
import meshzms;

import std.stdio;

class Heightmap : Resource {
public:
    override void load(char[] theFilename) {
        myFilename = theFilename;

        File hm = new File();
        hm.open(theFilename, FileMode.In);
        hm.readf(&heightmapFilename, &mySizeX, &mySizeY, &mySizeZ);
        hm.readf(&alphamapFilename, &tex1Filename, &tex2Filename, &tex3Filename);
        hm.readf(&repeatTex1, &repeatTex2, &repeatTex3);
        hm.readf(&meshGreenFilename, &mySizeMeshGreen);
        hm.close();

        Image   myImage = cast(Image)ResourceManager.getInstance.need(Image.classinfo, heightmapFilename);

        uint  p;
        ubyte*  r   = new ubyte();
        ubyte*  g  	= new ubyte();
        ubyte*  b  	= new ubyte();
        ubyte*  a  	= new ubyte();

        myWidth     =   myImage.width;
        myHeight    =   myImage.height;

        myVertizes.length   =   myWidth * myHeight;
        uvCoords[0].length  =   myWidth * myHeight;
        uvCoords[1].length  =   myWidth * myHeight;
        uvCoords[2].length  =   myWidth * myHeight;
        uvCoords[3].length  =   myWidth * myHeight;

        for (int x=0; x<myWidth; x++) {
            for (int y=0; y<myHeight; y++) {
                p = get_pixel(myImage.surface, x, y);
                SDL_GetRGBA(p, myImage.surface.format, r, g, b, a);

                myVertizes[x*myHeight+y].position.x = cast(float)x * mySizeX;
                myVertizes[x*myHeight+y].position.y = cast(float)y * mySizeY;
                myVertizes[x*myHeight+y].position.z = cast(float)*r * mySizeZ;

                uvCoords[0][x*myHeight+y].x = cast(float)x / myWidth;
                uvCoords[0][x*myHeight+y].y = cast(float)y / myHeight;

                uvCoords[1][x*myHeight+y].x = cast(float)x / myWidth * repeatTex1;
                uvCoords[1][x*myHeight+y].y = cast(float)y / myWidth * repeatTex1;

                uvCoords[2][x*myHeight+y].x = cast(float)x / myWidth * repeatTex2;
                uvCoords[2][x*myHeight+y].y = cast(float)y / myWidth * repeatTex2;

                uvCoords[3][x*myHeight+y].x = cast(float)x / myWidth * repeatTex3;
                uvCoords[3][x*myHeight+y].y = cast(float)y / myWidth * repeatTex3;
            };
        };

        calcNormals();

        calcVertexArray();

		if (ARBMultitexture.isEnabled() && ARBShaderObjects.isEnabled()) {
			myShader    =   new HLSLShader;
			myShader.load("resources/shaders/vertexshader_terrain.fx", "resources/shaders/fragmentshader_terrain.fx");
		};

        myTexture0 = cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, alphamapFilename);
        myTexture1 = cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, tex1Filename);
        myTexture2 = cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, tex2Filename);
        myTexture3 = cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, tex3Filename);

        myMeshGreen 	= cast(MeshZMS)ResourceManager.getInstance.need(MeshZMS.classinfo, meshGreenFilename);
        for (int y=myHeight-1; y>=0; y--) {
			for (int x=0; x<myWidth; x++) {
                p = get_pixel(myImage.surface, x, y);
                SDL_GetRGBA(p, myImage.surface.format, r, g, b, a);
                if (*g > 25) {
					myMeshGreenPositions.length = myMeshGreenPositions.length+1;
					myMeshGreenSizes.length 	= myMeshGreenSizes.length+1;

					myMeshGreenPositions[length-1].x = cast(float)x * mySizeX;
					myMeshGreenPositions[length-1].y = cast(float)y * mySizeY;
					myMeshGreenPositions[length-1].z = cast(float)*r * mySizeZ;

					myMeshGreenSizes[length-1] = cast(float)*g*mySizeZ*mySizeMeshGreen;
                };
            };
        };
        ResourceManager.getInstance.unNeed(myImage);
	};

    override void unload() {
    	if (ARBMultitexture.isEnabled() && ARBShaderObjects.isEnabled()) {
			myShader.unload();
			delete myShader;
    	};
        ResourceManager.getInstance.unNeed(myTexture0);
        ResourceManager.getInstance.unNeed(myTexture1);
        ResourceManager.getInstance.unNeed(myTexture2);
        ResourceManager.getInstance.unNeed(myTexture3);
        ResourceManager.getInstance.unNeed(myMeshGreen);
    };

    void draw(Point3D thePoint) {
        int start, end;
        if (cast(int)thePoint.x+drawLength > 0) start = 0;
        else start = std.math.abs(cast(int)thePoint.x+drawLength);
        if (std.math.abs(cast(int)thePoint.x-drawLength) >= cast(int)(mySizeX*myWidth) - 1) end = cast(int)(mySizeX*myWidth) - 1;
        else end = std.math.abs(cast(int)thePoint.x-drawLength);

        int startVertex = cast(int)(start/mySizeX);
        int endVertex = cast(int)(end/mySizeX);

		if (ARBMultitexture.isEnabled() && ARBShaderObjects.isEnabled()) {
			myShader.enableShader();
			for (int x=startVertex; x<endVertex; x+=1) {
				glClientActiveTextureARB(GL_TEXTURE0_ARB);
				glActiveTextureARB(GL_TEXTURE0_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture0.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[0].ptr );

				glClientActiveTextureARB(GL_TEXTURE1_ARB);
				glActiveTextureARB(GL_TEXTURE1_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture1.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[1].ptr );

				glClientActiveTextureARB(GL_TEXTURE2_ARB);
				glActiveTextureARB(GL_TEXTURE2_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture2.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[2].ptr );

				glClientActiveTextureARB(GL_TEXTURE3_ARB);
				glActiveTextureARB(GL_TEXTURE3_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture3.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[3].ptr );

				glDrawElements(GL_TRIANGLE_STRIP,myHeight*2, GL_UNSIGNED_INT, myIndices[x*myHeight*2..(x*myHeight*2)+myHeight*2].ptr );
			};

			glClientActiveTextureARB(GL_TEXTURE3_ARB);
			glActiveTextureARB(GL_TEXTURE3_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			glClientActiveTextureARB(GL_TEXTURE2_ARB);
			glActiveTextureARB(GL_TEXTURE2_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			glClientActiveTextureARB(GL_TEXTURE1_ARB);
			glActiveTextureARB(GL_TEXTURE1_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			glClientActiveTextureARB(GL_TEXTURE0_ARB);
			glActiveTextureARB(GL_TEXTURE0_ARB);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			myShader.disableShader();
        } else {
        	for (int x=startVertex; x<endVertex; x+=1) {
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture1.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[1].ptr );

				glDrawElements(GL_TRIANGLE_STRIP,myHeight*2, GL_UNSIGNED_INT, myIndices[x*myHeight*2..(x*myHeight*2)+myHeight*2].ptr );
			};

			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);
        }

		glEnable(GL_BLEND);
    	for(int i=0; i<myMeshGreenPositions.length && i<myMeshGreenSizes.length; i++) {
			if (myMeshGreenPositions[i].x > start) {
				if (myMeshGreenPositions[i].x < end) {
					glPushMatrix();

					glTranslatef( myMeshGreenPositions[i].x, myMeshGreenPositions[i].y, myMeshGreenPositions[i].z );
					glScalef(myMeshGreenSizes[i], myMeshGreenSizes[i], myMeshGreenSizes[i]);
					myMeshGreen.draw();

					glPopMatrix();
				};
			};
    	};
    	glDisable(GL_BLEND);

        //show Points
        /*glPointSize(10.0f);
        glBegin(GL_POINTS);
        foreach(vert; myVertizes) {
            glNormal3f(vert.normal.x, vert.normal.y, vert.normal.z);
            glVertex3f(vert.position.x, vert.position.y, vert.position.z);
        };
        glEnd();*/

        //show Normals
        /*for (int i=0; i<myWidth*myHeight; i++) {
            glBegin(GL_LINES);
                glVertex3f( myVertizes[i].position.x+myVertizes[i].normal.x, myVertizes[i].position.y+myVertizes[i].normal.y, myVertizes[i].position.z+myVertizes[i].normal.z );
                glVertex3f( myVertizes[i].position.x, myVertizes[i].position.y, myVertizes[i].position.z );
            glEnd();
        };*/

    };

    void draw() {
        if (ARBMultitexture.isEnabled() && ARBShaderObjects.isEnabled()) {
			myShader.enableShader();
			for (int x=0; x<myWidth-1; x+=1) {
				glClientActiveTextureARB(GL_TEXTURE0_ARB);
				glActiveTextureARB(GL_TEXTURE0_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture0.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[0].ptr );

				glClientActiveTextureARB(GL_TEXTURE1_ARB);
				glActiveTextureARB(GL_TEXTURE1_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture1.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[1].ptr );

				glClientActiveTextureARB(GL_TEXTURE2_ARB);
				glActiveTextureARB(GL_TEXTURE2_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture2.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[2].ptr );

				glClientActiveTextureARB(GL_TEXTURE3_ARB);
				glActiveTextureARB(GL_TEXTURE3_ARB);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture3.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[3].ptr );

				glDrawElements(GL_TRIANGLE_STRIP,myHeight*2, GL_UNSIGNED_INT, myIndices[x*myHeight*2..(x*myHeight*2)+myHeight*2].ptr );
			};

			glClientActiveTextureARB(GL_TEXTURE3_ARB);
			glActiveTextureARB(GL_TEXTURE3_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			glClientActiveTextureARB(GL_TEXTURE2_ARB);
			glActiveTextureARB(GL_TEXTURE2_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			glClientActiveTextureARB(GL_TEXTURE1_ARB);
			glActiveTextureARB(GL_TEXTURE1_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			glClientActiveTextureARB(GL_TEXTURE0_ARB);
			glActiveTextureARB(GL_TEXTURE0_ARB);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);

			myShader.disableShader();
        } else {
        	for (int x=0; x<myWidth-1; x+=1) {
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, myTexture1.texture);
				glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glEnableClientState(GL_VERTEX_ARRAY);
				glEnableClientState(GL_NORMAL_ARRAY);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);

				glVertexPointer( 3, GL_FLOAT, Vertex.sizeof, myVertizes.ptr );
				glNormalPointer(GL_FLOAT, Vertex.sizeof, &myVertizes[0].normal );
				glTexCoordPointer(2, GL_FLOAT, Point2D.sizeof, uvCoords[1].ptr );

				glDrawElements(GL_TRIANGLE_STRIP,myHeight*2, GL_UNSIGNED_INT, myIndices[x*myHeight*2..(x*myHeight*2)+myHeight*2].ptr );
			};

			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_VERTEX_ARRAY);
        }

        glEnable(GL_BLEND);
    	for(int i=0; i<myMeshGreenPositions.length && i<myMeshGreenSizes.length; i++) {
			glPushMatrix();

			glTranslatef( myMeshGreenPositions[i].x, myMeshGreenPositions[i].y, myMeshGreenPositions[i].z );
			glScalef(myMeshGreenSizes[i], myMeshGreenSizes[i], myMeshGreenSizes[i]);
			myMeshGreen.draw();

			glPopMatrix();
    	};
    	glDisable(GL_BLEND);
    };

    float   getHeight(float x, float y) {
        x /= mySizeX;
        y /= mySizeY;
        if (x >= myWidth-1) x = myWidth-2;
        if (x < 0) x = 0;
        if (y >= myHeight-1) y = myHeight-2;
        if (y < 0) y = 0;
        int indexX = cast(int)x;
        int indexY = cast(int)y;
        float xd = x-indexX;
        float yd = y-indexY;
        Vertex temp = myVertizes[indexX*myHeight+indexY];
        return temp.position.z + xd * (myVertizes[(indexX+1)*myHeight+indexY].position.z - temp.position.z) + yd * (myVertizes[(indexX)*myHeight+indexY+1].position.z - temp.position.z);   // - xd*temp.normal.x + yd*temp.normal.y;
    };

    float sizeX() { return mySizeX*myWidth; };
    float sizeY() { return mySizeY*myHeight; };
    float sizeZ() { return mySizeZ*255.0f; };

private:

    Point3D calcNormal(Point3D  theP1, Point3D theP2, Point3D theP3) {
        Point3D tempA   =   theP2   -   theP1;
        Point3D tempB   =   theP3   -   theP1;
        Point3D result  =   Point3D.cross(tempA, tempB);
        result.normalize();
        return result;
    };

    void calcNormals() {
        myNormals.length    =   (myWidth-1)*2*(myHeight-1);
        for(int x=0; x<myWidth-1; x++) {
            for(int y=1; y<myHeight; y++) {
                myNormals[2*x*(myHeight-1)+(y-1)*2]   = calcNormal(myVertizes[x*myHeight+y].position, myVertizes[x*myHeight+y-1].position, myVertizes[(x+1)*myHeight+y-1].position );
                myNormals[2*x*(myHeight-1)+(y-1)*2+1] = calcNormal(myVertizes[x*myHeight+y].position, myVertizes[(x+1)*myHeight+y-1].position, myVertizes[(x+1)*myHeight+y].position );
            };
        };
        for(int x=0; x<myWidth;x++) {
            for(int y=0; y<myHeight;y++) {
                if (0>=x || myWidth-1<=x || 0>=y || myHeight-1<=y) { // rand
                    myVertizes[x*myHeight+y].normal.x = 0.0f;
                    myVertizes[x*myHeight+y].normal.y = 0.0f;
                    myVertizes[x*myHeight+y].normal.z = 1.0f;
                } else {
                    myVertizes[x*myHeight+y].normal = ( myNormals[(myHeight-1)*x*2+1+(y-1)*2]     + myNormals[(myHeight-1)*x*2+(y-1)*2+2]     + myNormals[(myHeight-1)*x*2+(y-1)*2+3]
                                                      + myNormals[(myHeight-1)*(x-1)*2+2+(y-1)*2] + myNormals[(myHeight-1)*(x-1)*2+3+(y-1)*2] + myNormals[(myHeight-1)*(x-1)*2+4+(y-1)*2] )
                                                      * (1.0f/6.0f);
                };
            };
        };
    };

    void calcVertexArray() {
        myIndices.length = myHeight * 2 * (myWidth-1);
        for (int x=0; x<myWidth-1; x++) {
            for (int y=0; y<myHeight; y++) {
                myIndices[2*x*myHeight+y*2]     = x*myHeight+y;
                myIndices[2*x*myHeight+y*2+1]   = x*myHeight+myHeight+y;
            };
        };
    };

    uint get_pixel(SDL_Surface *surface, int x, int y)
    {
        int bpp = surface.format.BytesPerPixel;
        Uint8 *p = cast(ubyte *)surface.pixels + y * surface.pitch + x * bpp;
        switch(bpp) {
        case 1:
            return *p;
        case 2:
            return *cast(ushort *)p;
        case 3:
            if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
                return p[0] << 16 | p[1] << 8 | p[2];
            else
                return p[0] | p[1] << 8 | p[2] << 16;
        case 4:
            return *cast(uint *)p;
        default: break;
        };
        return 0;
    };

    char[]      heightmapFilename;
    char[]      alphamapFilename;
    char[]      tex1Filename;
    char[]      tex2Filename;
    char[]      tex3Filename;
    char[]		meshGreenFilename;
    float       mySizeX;
    float       mySizeY;
    float       mySizeZ;
    float		mySizeMeshGreen;
    float       repeatTex1;
    float       repeatTex2;
    float       repeatTex3;

    uint        displayList;

    uint[]      myIndices;
    Vertex[]    myVertizes;
    Point2D[][4] uvCoords;
    Point3D[]   myNormals;

    Point3D[]	myMeshGreenPositions;
    float[]		myMeshGreenSizes;

    MeshZMS		myMeshGreen;
    HLSLShader  myShader;
    Texture2D   myTexture0;
    Texture2D   myTexture1;
    Texture2D   myTexture2;
    Texture2D   myTexture3;

    int         myWidth;
    int         myHeight;

    int         drawLength = 70;
};

