/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 6.1.2008
 *
 * This is the Shader class.
 */
module hlslshader;

import derelict.opengl.gl;
import derelict.opengl.extension.arb.shader_objects;

import std.stream;
import resource;

class HLSLShader : Resource {

    void load( char[] vertexshader = "", char[] fragmentshader = "" ) {
        int param;
        if (vertexshader.length > 0 || fragmentshader.length > 0) program = glCreateProgramObjectARB();

        if (vertexshader.length > 0) {
            char* stringVertex   = readFX(vertexshader);
            uint vertexProgram   = glCreateShaderObjectARB(GL_VERTEX_SHADER);
            if (vertexProgram==0) throw new Exception(__FILE__ ~ ": HLSLShader error: cannot create Vertex Shader.");
            glShaderSourceARB(vertexProgram, 1, &stringVertex, null);
            glCompileShaderARB(vertexProgram);

            glGetObjectParameterivARB(vertexProgram,  GL_OBJECT_COMPILE_STATUS_ARB, &param);
            if (!param) throw new Exception(__FILE__ ~ ": HLSLShader error: There is an error in the 'Vertex' Shader Source.");

            glAttachObjectARB(program, vertexProgram);
            glDeleteObjectARB(vertexProgram);
        };

        if (fragmentshader.length > 0) {
            char* stringFragment = readFX(fragmentshader);
            uint fragmentProgram = glCreateShaderObjectARB(GL_FRAGMENT_SHADER);
            if (fragmentProgram==0) throw new Exception(__FILE__ ~ ": HLSLShader error: cannot create Fragment Shader.");
            glShaderSourceARB(fragmentProgram, 1, &stringFragment, null);
            glCompileShaderARB(fragmentProgram);

            glGetObjectParameterivARB(fragmentProgram,  GL_OBJECT_COMPILE_STATUS_ARB, &param);
            if (!param) throw new Exception(__FILE__ ~ ": HLSLShader error: There is an error in the 'Fragment' Shader Source.");

            glAttachObjectARB(program, fragmentProgram);
            glDeleteObjectARB(fragmentProgram);
        };

        if (vertexshader.length > 0 || fragmentshader.length > 0) {
            glLinkProgramARB(program);
            glGetObjectParameterivARB(program, GL_OBJECT_LINK_STATUS_ARB, &param);
            if (!param) throw new Exception(__FILE__ ~ ": HLSLShader error: It was not possible to Link the Shader Program.");
        };

        glUseProgramObjectARB(program);
        glUniform1iARB(glGetUniformLocationARB(program, "tex0"), 0);
        glUniform1iARB(glGetUniformLocationARB(program, "tex1"), 1);
        glUniform1iARB(glGetUniformLocationARB(program, "tex2"), 2);
        glUniform1iARB(glGetUniformLocationARB(program, "tex3"), 3);
        glUseProgramObjectARB(0);
    };

    void unload() {
        glUseProgramObjectARB(0);
        glDeleteObjectARB(program);
    };

    void enableShader() {
        glUseProgramObjectARB(program);

    };
    void disableShader() {
        glUseProgramObjectARB(0);
    };



private:

    char* readFX(char[] filename) {
        File myFile = new File;
        char[] temp;
        myFile.open(filename, FileMode.In);
        while(!myFile.eof) {
            temp ~= myFile.readLine();
            temp ~= "\n";
        };
        myFile.close();
        return std.string.toStringz(temp);
    };

    uint    program;

};
