/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 17.1.2007
 *
 * This is Image Resource: It can load BMP, JPG, PNG... Additional types can be added easily.
 */
module image;

import derelict.sdl.sdl;
import derelict.sdl.image;
import derelict.opengl.gl;

import resource;

class Image : Resource {

public:
    override void loadSpecific(char[] theFilename) {
        myFilename      =   theFilename;
        SDL_RWops *file	=	SDL_RWFromFile( std.string.toStringz(theFilename), "rb");
        if (file !is null)
        {
            if(IMG_isBMP(file)) {
				mySurface   = IMG_LoadBMP_RW(file);
				myIformat   = GL_RGB;
                myFormat    = GL_BGR;
			} else if(IMG_isJPG(file)) {
				mySurface   = IMG_LoadJPG_RW(file);
				myIformat   = GL_RGB;
                myFormat 	= GL_BGR;
			} else if(IMG_isPNG(file)) {
				mySurface   = IMG_LoadPNG_RW(file);
				if (mySurface.format.Amask!=0) {
			        myIformat   = GL_RGBA;
                    myFormat    = GL_RGBA;
			    } else {
                    myIformat   = GL_RGB;
                    myFormat 	= GL_RGB;
			    };
			} else {
                SDL_FreeRW(file);
                delete file;
			    throw new UnloadableResourceException(__FILE__ ~ ": Cannot load Resource " ~ theFilename ~ ". Unknown type.");
			};
			myHeight  = mySurface.h;
			myWidth   = mySurface.w;
			SDL_FreeRW(file);
			delete file;
		} else {
		    throw new UnloadableResourceException(__FILE__ ~ ": Cannot load Resource " ~ theFilename ~ ". Cannot find that file.");
		};
    };
    override void unloadSpecific() {
        if (mySurface) {
            SDL_FreeSurface(mySurface);
            mySurface =   null;
        };
    };
    override void loadDefault() {
        loadSpecific("resources/textures/standardpic.png");
    };

    int width()     { return myWidth; };
    int height()    { return myHeight; };
    int format()    { return myFormat; };
    int iformat()   { return myIformat; };

    SDL_Surface*    surface() { return mySurface; };

private:

    int myWidth;
	int myHeight;
	int myFormat;
	int myIformat;

	SDL_Surface*    mySurface;
};
