/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 2.1.2008
 *
 * The Input Module handles the Users Input with SDL
 */
module input;

import derelict.sdl.sdl;
import singleton;
import task;

class Input : Task {
public:
    mixin Singleton;
    override void init() { SDL_EnableUNICODE(1); };
    override int refresh() {
        while(SDL_PollEvent(&myEvent))
		{
		    switch(myEvent.type)
		    {
		         case SDL_MOUSEMOTION:
                    myMouse_x	=	myEvent.motion.x;
                    myMouse_y	=	myEvent.motion.y;
                    break;
		        case SDL_KEYDOWN:
                    myKeys.length = myKeys.length+1;
                    myKeys[myKeys.length-1].sdl_key = myEvent.key.keysym.sym;
                    myKeys[myKeys.length-1].unicode = myEvent.key.keysym.unicode;
                    break;
                case SDL_KEYUP:
                    for(int i=0; i<myKeys.length; i++) {
                        if (myKeys[i].sdl_key == myEvent.key.keysym.sym) {
                            if (myKeys[i] != myKeys[length-1] ) myKeys[i] = myKeys[length-1];
                            myKeys.length = myKeys.length - 1;
                            break;
                        };
                    };
                    break;
                case SDL_MOUSEBUTTONUP:
                    switch(myEvent.button.button)
                    {
                    case SDL_BUTTON_LEFT:
                        myMouseLeft = 0;
                        break;
                    case SDL_BUTTON_MIDDLE:
                        myMouseMiddle = 0;
                        break;
                    case SDL_BUTTON_RIGHT:
                        myMouseRight = 0;
                        break;
                    case SDL_BUTTON_WHEELUP:
                        //myMouseWheelUp = 0;
                        break;
                    case SDL_BUTTON_WHEELDOWN:
                        //myMouseWheelDown = 0;
                        break;
                    default: break;
                    }
                    break;
                case SDL_MOUSEBUTTONDOWN:
                    switch(myEvent.button.button)
                    {
                    case SDL_BUTTON_LEFT:
                        myMouseLeft = 1;
                        break;
                    case SDL_BUTTON_MIDDLE:
                        myMouseMiddle = 1;
                        break;
                    case SDL_BUTTON_RIGHT:
                        myMouseRight = 1;
                        break;
                    case SDL_BUTTON_WHEELUP:
                        myMouseWheelUp = 1;
                        break;
                    case SDL_BUTTON_WHEELDOWN:
                        myMouseWheelDown = 1;
                        break;
                    default: break;
                    }
                    break;
                case SDL_QUIT:
                    myKeys.length = myKeys.length+1;
                    myKeys[myKeys.length-1].sdl_key = SDLK_ESCAPE;
                    myKeys[myKeys.length-1].unicode = SDLK_ESCAPE;
                    break;
                default: break;
		    };
		};
		return 0;
    };
    override void uninit() {};
    Key[]   keys()              { return myKeys;            };
    int     mouse_x()           { return myMouse_x;         };
    int		mouse_y()           { return myMouse_y;         };
	int		mouseLeft()         { return myMouseLeft;       };
	int		mouseMiddle()       { return myMouseMiddle;     };
	int		mouseRight()        { return myMouseRight;      };
	int		mouseWheelUp()      { return myMouseWheelUp;    };
	int		mouseWheelDown()    { return myMouseWheelDown;  };
	void	mouseWheelUpReset()   { myMouseWheelUp   = 0;   };
	void	mouseWheelDownReset() { myMouseWheelDown = 0;   };

private:
    struct Key
    {
        int	sdl_key;
        int unicode;
    }
    SDL_Event 	myEvent;
    Key[]	    myKeys;
    int		    myMouse_x;
	int		    myMouse_y;
	int		    myMouseLeft;
	int		    myMouseMiddle;
	int		    myMouseRight;
	int		    myMouseWheelUp;
	int		    myMouseWheelDown;
};
