/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 6.1.2008
 *
 * the menue interface
 */
module menue;

import derelict.opengl.gl;
import renderer;
import actionlistener;
import task;
import timer;
import point;
import appstate;
import resourcemanager;
import heightmap;
import texture2d;
import sound;

import menueimpl;
import menuemain;
import menuelvl;
import menuecopyright;
import menuelost;
import menuewonbronze;
import menuewonsilver;
import menuewongold;
import actionlistener;


import std.stdio;

class Menue : ActionListener {
    void init(APP_STATE theState) {
        myHeightmap 	= cast(Heightmap)ResourceManager.getInstance.need(Heightmap.classinfo, "resources/heightmaps/start.txt");
        myTexture   	= cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, "resources/textures/menue_shade.png");
        myClickSound	= cast(Sound)ResourceManager.getInstance().need((Sound.classinfo), "resources/sounds/menue_click.wav");
        myOverSound		= cast(Sound)ResourceManager.getInstance().need((Sound.classinfo), "resources/sounds/menue_over.wav");


        pos.x = 0;
        pos.y = -20;
        pos.z = -220;

        rot.x = 90.0f;
        rot.y = -80.0f;

        myState = theState;
    };

    int refresh() {
		Renderer.getInstance().join3D();
        rot.x += 0.005f*cast(float)Timer.getInstance().ticks;
        glTranslatef(pos.x,pos.y, pos.z);
        glRotatef(rot.y,1,0,0);
        glRotatef(rot.x,0,0,1);
        glTranslatef(-190,-190,0);
        myHeightmap.draw();

        Renderer.getInstance().join2D();
        glEnable(GL_BLEND);
        glLoadIdentity();
        glBindTexture(GL_TEXTURE_2D, myTexture.texture);
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
        glBegin(GL_QUADS);
            glTexCoord2f(0,0.21f);   glVertex2f(Renderer.getInstance().width, 0);
            glTexCoord2f(1,0.21f);   glVertex2f(0,                            0);
            glTexCoord2f(1,0.40f);   glVertex2f(0,                            100);
            glTexCoord2f(0,0.40f);   glVertex2f(Renderer.getInstance().width, 100);
        glEnd();
        glBegin(GL_QUADS);
            glTexCoord2f(0,0);       glVertex2f( Renderer.getInstance().width-512.0f, Renderer.getInstance().height);
            glTexCoord2f(1,0);       glVertex2f( Renderer.getInstance().width,        Renderer.getInstance().height);
            glTexCoord2f(1,0.2f);    glVertex2f( Renderer.getInstance().width,        Renderer.getInstance().height-100);
            glTexCoord2f(0,0.2f);    glVertex2f( Renderer.getInstance().width-512.0f, Renderer.getInstance().height-100);
        glEnd();
        glBegin(GL_QUADS);
            glTexCoord2f(0,   0);    glVertex2f( Renderer.getInstance().width-512.0f, Renderer.getInstance().height);
            glTexCoord2f(0.1f,0);    glVertex2f( 0,                                   Renderer.getInstance().height);
            glTexCoord2f(0.1f,0.2f); glVertex2f( 0,                                   Renderer.getInstance().height-100);
            glTexCoord2f(0,   0.2f); glVertex2f( Renderer.getInstance().width-512.0f, Renderer.getInstance().height-100);
        glEnd();
        glDisable(GL_BLEND);

        switch(myState) {
            case APP_STATE.MENUE_RUNNING:
                myMenueImpl.refresh();
                break;
            case APP_STATE.STARTIMPL_MAIN:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueMain();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
             case APP_STATE.STARTIMPL_LVL:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueLVL();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
             case APP_STATE.STARTIMPL_COPYRIGHT:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueCopyright();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
            case APP_STATE.STARTIMPL_LOST:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueLost();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
            case APP_STATE.STARTIMPL_WON_BRONZE:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueWonBronze();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
            case APP_STATE.STARTIMPL_WON_SILVER:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueWonSilver();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
            case APP_STATE.STARTIMPL_WON_GOLD:
                if (myMenueImpl) {
                    myMenueImpl.uninit();
                    delete myMenueImpl;
                };
                myMenueImpl = new MenueWonGold();
                myMenueImpl.init(this);
                myState = APP_STATE.MENUE_RUNNING;
                break;
            case APP_STATE.QUIT:
                myActionListener.actionPerformed("Menue", "ShutDown");
                break;
        };

        return myState;
    };

    void uninit() {
        ResourceManager.getInstance.unNeed(myHeightmap);
        ResourceManager.getInstance.unNeed(myTexture);
        ResourceManager.getInstance.unNeed(myClickSound);
        ResourceManager.getInstance.unNeed(myOverSound);
    };

    void actionPerformed(char[] command, char[] attribute) {
        switch(command) {
            case "Menue":
                switch(attribute) {
                    case "LVL":
                        myState = APP_STATE.STARTIMPL_LVL;
                        break;
                    case "Copyright":
                        myState = APP_STATE.STARTIMPL_COPYRIGHT;
                        break;
                    case "Main":
                        myState = APP_STATE.STARTIMPL_MAIN;
                        break;
                    case "Quit":
                    default:
                        myState = APP_STATE.QUIT;
                        break;
                };
            break;
            case "Game":
                myActionListener.actionPerformed(command, attribute);
            break;
            default: break;
        };
    };
    void addActionListener(ActionListener al, char[] theCommand="", char[] theAttribute="") {
        myActionListener    = al;
        command             = theCommand;
        attribute           = theAttribute;
    };

protected:
    Heightmap       myHeightmap;
    Texture2D       myTexture;
    Sound			myClickSound;
    Sound			myOverSound;

    Point3D         pos;
    Point2D         rot;

    MenueImpl       myMenueImpl;
    ActionListener  myActionListener;
    char[]          command;
    char[]          attribute;

    APP_STATE       myState;
};
