/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 11.1.2008
 *
 * This is the LVL Menue!
 */
module menuelvl;

import derelict.opengl.gl;
import menueimpl;
import button;
import imagebutton;
import config;
import point;
import resourcemanager;
import texture2d;

import std.stream;

import std.stdio;

class MenueLVL : MenueImpl {
public:
    override void init(Menue theMenue) {
        myButtons.length = 1;
        myButtons[0] = new Button;
        myButtons[0].init(Point2D(Config["window_width"]-250, 120), Point2D(256,37), ".main menu", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myButtons[0].addActionListener(theMenue, "Menue", "Main");

        myImageButtons.length = 7;
        myImageButtons[0] = new ImageButton();
        myImageButtons[0].init(Point2D(50, Config["window_height"]-250), Point2D(128,128),"Level 1", "resources/textures/lvls/lvl1_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[0].addActionListener(theMenue, "Game", "resources/lvls/lvl1.txt");

        myImageButtons[1] = new ImageButton();
        myImageButtons[1].init(Point2D(250, Config["window_height"]-250), Point2D(128,128),"Level 2", "resources/textures/lvls/lvl2_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[1].addActionListener(theMenue, "Game", "resources/lvls/lvl2.txt");

        myImageButtons[2] = new ImageButton();
        myImageButtons[2].init(Point2D(450, Config["window_height"]-250), Point2D(128,128),"Level 3", "resources/textures/lvls/lvl3_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[2].addActionListener(theMenue, "Game", "resources/lvls/lvl3.txt");

        myImageButtons[3] = new ImageButton();
        myImageButtons[3].init(Point2D(250, Config["window_height"]-650), Point2D(128,128),"Special Level", "resources/textures/lvls/lvl4_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[3].addActionListener(theMenue, "Game", "resources/lvls/lvlSpecial.txt");

        myImageButtons[4] = new ImageButton();
        myImageButtons[4].init(Point2D(50, Config["window_height"]-450), Point2D(128,128),"Level 4", "resources/textures/lvls/lvl5_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[4].addActionListener(theMenue, "Game", "resources/lvls/lvl1b.txt");

        myImageButtons[5] = new ImageButton();
        myImageButtons[5].init(Point2D(250, Config["window_height"]-450), Point2D(128,128),"Level 5", "resources/textures/lvls/lvl6_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[5].addActionListener(theMenue, "Game", "resources/lvls/lvl2b.txt");

        myImageButtons[6] = new ImageButton();
        myImageButtons[6].init(Point2D(450, Config["window_height"]-450), Point2D(128,128),"Level 6", "resources/textures/lvls/lvl7_thumb.png", "resources/sounds/menue_click.wav", "resources/sounds/menue_over.wav");
        myImageButtons[6].addActionListener(theMenue, "Game", "resources/lvls/lvl3b.txt");

        File hm = new File();
        hm.open("resources/savegame", FileMode.In);
        int data;
        while (!hm.eof) {
            hm.readf(&data);
            savegame ~= data;
        };
        hm.close();

        myTexGold   = cast(Texture2D)ResourceManager.getInstance().need((Texture2D.classinfo), "resources/textures/prize/gold.png");
        myTexSilver = cast(Texture2D)ResourceManager.getInstance().need((Texture2D.classinfo), "resources/textures/prize/silver.png");
        myTexBronze = cast(Texture2D)ResourceManager.getInstance().need((Texture2D.classinfo), "resources/textures/prize/bronze.png");
    };
    override void refresh() {
        foreach(button; myButtons) button.refresh();

        for (int i=0; i<myImageButtons.length && i<savegame.length; i++) {
            myImageButtons[i].refresh();
            switch(savegame[i]) {
                case 0:
                break;
                case 1:
                    glBindTexture(GL_TEXTURE_2D, myTexBronze.texture);
                    drawPrize(i);
                break;
                case 2:
                    glBindTexture(GL_TEXTURE_2D, myTexSilver.texture);
                    drawPrize(i);
                break;
                default:
                    glBindTexture(GL_TEXTURE_2D, myTexGold.texture);
                    drawPrize(i);
                break;
            };
        };
    };
    override void uninit() {
        foreach(button; myButtons) {
            button.uninit();
            delete button;
        };
        foreach(button; myImageButtons) {
            button.uninit();
            delete button;
        };
        ResourceManager.getInstance().unNeed(myTexGold);
        ResourceManager.getInstance().unNeed(myTexSilver);
        ResourceManager.getInstance().unNeed(myTexBronze);
    };

private:
    void drawPrize(int i) {
        glEnable(GL_BLEND);
        glLoadIdentity();
        glTranslatef(myImageButtons[i].position.x + myImageButtons[i].dimension.x-32, myImageButtons[i].position.y-32, 0);
        glBegin(GL_QUADS);
            glTexCoord2f(0,1); glVertex2f(0,   0);
            glTexCoord2f(1,1); glVertex2f(64,  0);
            glTexCoord2f(1,0); glVertex2f(64,  64);
            glTexCoord2f(0,0); glVertex2f(0,   64);
        glEnd();
        glDisable(GL_BLEND);
    };

    Button[]        myButtons;
    ImageButton[]   myImageButtons;
    int[]           savegame;
    Texture2D       myTexGold;
    Texture2D       myTexSilver;
    Texture2D       myTexBronze;
};


