/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 05.05.2008
 *
 * This is the Mesh Class. ZMS: Zweistein Model Static
 */
module meshzms;

import derelict.opengl.gl;

import resourcemanager;
import resource;
import texture2d;

import point;
import face;
import material;
import vertex;

import std.stream;

import std.stdio;
import input;

class MeshZMS : Resource {
public:
    override void loadSpecific(char[] theFilename) {
        myFilename     = theFilename;

        char    newLine;
        int     mode, smooth, index, count, vcount;

        File ZMSFile = new File();
        ZMSFile.open(theFilename, FileMode.In);
        ZMSFile.readf("%d %d %d %d", &hasMaterial, &vertizesCount, &facesCount, &hasUV);

        myVertizes.length   = vertizesCount;
        myFaces.length      = facesCount;

        if (hasUV) {
            char[]  filenameUV;
            ZMSFile.readf(&filenameUV);
            myTexture = cast(Texture2D)ResourceManager.getInstance.need(Texture2D.classinfo, filenameUV);
        };
        if (!hasMaterial) mode++;

        while(!ZMSFile.eof()) {
            switch(mode) {
                case 0: // Material reading
                    ZMSFile.readf(&myMaterial.r, &myMaterial.g, &myMaterial.b, &myMaterial.spec, &myMaterial.hard);
                    mode++;
                break;
                case 1: // Vertizes reading
                    ZMSFile.readf( &myVertizes[count].normal.x, &myVertizes[count].normal.y, &myVertizes[count].normal.z,
                                   &myVertizes[count].position.x, &myVertizes[count].position.y, &myVertizes[count].position.z );
                    count++;
                    if (count >= vertizesCount) {
                        mode++;
                        count = 0;
                    };
                break;
                case 2: // Face and UV Reading
                    if (count < facesCount) {
                        ZMSFile.readf( &smooth, &myFaces[count].faceNormal.x, &myFaces[count].faceNormal.y, &myFaces[count].faceNormal.z, &vcount);
                        myFaces[count].isSmooth = cast(bool)smooth;

                        for (int i=0; i<vcount; i++) {
                            myFaces[count].vertexIDs.length = vcount;
                            ZMSFile.readf( &myFaces[count].vertexIDs[i] );
                            if (hasUV) {
                                myFaces[count].uvCoords.length = myFaces[count].uvCoords.length+1;
                                ZMSFile.readf( &myFaces[count].uvCoords[length-1].x, &myFaces[count].uvCoords[length-1].y );
                            };
                        };
                        count++;
                        if (count >= facesCount) mode++;
                    };
                break;
                default:
                    ZMSFile.readf(&newLine);
                break;
            };
        };
        ZMSFile.close();
    };

    void loadDefault() {
		loadSpecific("resources/meshes/defaultmesh.zms");
	};

    override void unloadSpecific() {
        ResourceManager.getInstance.unNeed(myTexture);
    };

    void draw() {
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        if (hasUV) {
            glBindTexture(GL_TEXTURE_2D, myTexture.texture);
        } else glDisable(GL_TEXTURE_2D);

        //glRotatef(Input.getInstance.mouse_x, 1,0,0);

        for (int i=0; i<myFaces.length; i++) {
            if (myFaces[i].vertexIDs.length == 3) glBegin(GL_TRIANGLES);
            else glBegin(GL_QUADS);

            if (!myFaces[i].isSmooth) glNormal3f(myFaces[i].faceNormal.x, myFaces[i].faceNormal.y, myFaces[i].faceNormal.z );

            for (int j=0; j<myFaces[i].vertexIDs.length; j++) {
                if (myFaces[i].isSmooth) glNormal3f(myVertizes[myFaces[i].vertexIDs[j]].normal.x, myVertizes[myFaces[i].vertexIDs[j]].normal.y, myVertizes[myFaces[i].vertexIDs[j]].normal.z );
                if (hasUV) glTexCoord2f(myFaces[i].uvCoords[j].x, myFaces[i].uvCoords[j].y );
                glVertex3f(myVertizes[myFaces[i].vertexIDs[j]].position.x, myVertizes[myFaces[i].vertexIDs[j]].position.y, myVertizes[myFaces[i].vertexIDs[j]].position.z );
            };

            glEnd();
        };
        glDisable(GL_LIGHTING);
        glDisable(GL_LIGHT0);
    };



private:
    int hasMaterial;
    int vertizesCount;
    int facesCount;
    int hasUV;

    uint displayList;

    Texture2D   myTexture;
    Material    myMaterial;
    Vertex[]    myVertizes;
    Face[]      myFaces;

};
