/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 6.1.2008
 *
 * The Player is the moveable element in the game
 */
module player;

import derelict.opengl.gl;
import derelict.sdl.mixer;

import point;
import timer;

import resourcemanager;
import meshzms;
import sound;

float PLAYER_SPEED = 0.08f;

class Player {
public:

    void init() {
        myMesh = cast(MeshZMS)ResourceManager.getInstance.need(MeshZMS.classinfo, "resources/meshes/player.zms");
        myJumpSound = cast(Sound)ResourceManager.getInstance.need(Sound.classinfo, "resources/sounds/player_jump.wav");
        myLandSound = cast(Sound)ResourceManager.getInstance.need(Sound.classinfo, "resources/sounds/player_land2.wav");
        myWalkSound = cast(Sound)ResourceManager.getInstance.need(Sound.classinfo, "resources/sounds/player_walk.wav");

    };
    void uninit() {
        ResourceManager.getInstance.unNeed(myMesh);
        ResourceManager.getInstance.unNeed(myWalkSound);
        ResourceManager.getInstance.unNeed(myLandSound);
        ResourceManager.getInstance.unNeed(myMesh);
    };

    void refresh() {
        if (jumping) {
            //if (isDown) v0 += 0.3f;
            t += Timer.getInstance().ticks * 0.01f;
                position.z  = jumpStartHeight + v0 * t - 0.5f * g * t * t;
                if (oldHeight > position.z) isDown = 1;
                oldHeight = position.z;
		};
		if (!walking2) {
			Mix_HaltChannel(15);
			walking = false;
		};
		walking2 = false;
	};

	void jump() {
		if (!isJumping) {
			Mix_PlayChannel(-1, myJumpSound.sound, 0);
			Mix_HaltChannel(15);
			walking = false;
			isJumping = 1;
            v0  = 9.5f;
            t   = 0.0f;
            g   = 2.0f;
            oldHeight = jumpStartHeight = position.z;
        };
        //v0 += 0.3f;
    };
    void walk() {
    	Mix_PlayChannel(-1, myLandSound.sound, 0);
		walking = false;
        isJumping   = 0;
        isDown = 0;
    };

    void draw() {
        glPushMatrix();
        glTranslatef(position.x, position.y, position.z);
        glEnable(GL_BLEND);
        myMesh.draw();
        glDisable(GL_BLEND);
        glPopMatrix();
    };

    void left() {
		if (!walking && !isJumping)	Mix_PlayChannel(15, myWalkSound.sound, -1);
		walking = true;
		walking2 = true;
		position.x -= PLAYER_SPEED * Timer.getInstance().ticks;
    };

    void right() {
    	if (!walking && !isJumping) Mix_PlayChannel(15, myWalkSound.sound, 1);
		walking = true;
		walking2 = true;
		position.x += PLAYER_SPEED * Timer.getInstance().ticks;
    };

    int jumping() { return isJumping; };
    int down() { return isDown; };

    Point3D position;

private:
    int     isJumping = 0;
    int     isDown = 0;
    float   jumpStartHeight;
    float   oldHeight;

    float   v0;
    float   t;
    float   g;

    bool 	walking;
    bool	walking2;

    MeshZMS 	myMesh;
    Sound		myJumpSound;
    Sound		myLandSound;
    Sound		myWalkSound;
};
