/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 2.1.2008
 *
 * This Point Module is for Points with 2 dimensions (Point2D(x,y)) or 3 dimensions (Point3D(x,y,z)).
 * There are some functionalities like: addition, scalar, multiplikation, length, normalize, crossproduct...
 */
module point;

import std.math: sqrt;

struct Point2D {
public:
    static Point2D opCall() {
        Point2D myPoint;
        return myPoint;
    }
    static Point2D opCall(float x, float y) {
        Point2D myPoint;
        myPoint.x = x;
        myPoint.y = y;
        return myPoint;
    }
    static Point2D opCall(int x, int y) {
        Point2D myPoint;
        myPoint.x = x;
        myPoint.y = y;
        return myPoint;
    }
    Point2D opAdd(Point2D thePoint) {
        Point2D result;
        result.x = x + thePoint.x;
        result.y = y + thePoint.y;
        return result;
    };
    Point2D opSub(Point2D thePoint) {
        Point2D result;
        result.x = x - thePoint.x;
        result.y = y - thePoint.y;
        return result;
    };
    float opMul(Point2D thePoint) {
        return (thePoint.x * x) + (thePoint.y * y);
    };
    Point2D opMul(float theS) {
        Point2D result;
        result.x = x * theS;
        result.y = y * theS;
        return result;
    };
    float   length() {
        return sqrt( this.x*this.x + this.y*this.y );
    };
    void normalize() {
        float length =  length();
        this.x = this.x / length;
        this.y = this.y / length;
    };
    float x;
    float y;
};

struct Point3D {
public:
    Point3D opAdd(Point3D thePoint) {
        Point3D result;
        result.x = x + thePoint.x;
        result.y = y + thePoint.y;
        result.z = z + thePoint.z;
        return result;
    };
    Point3D opSub(Point3D thePoint) {
        Point3D result;
        result.x = x - thePoint.x;
        result.y = y - thePoint.y;
        result.z = z - thePoint.z;
        return result;
    };
    float opMul(Point3D thePoint) {
        return (thePoint.x * x) + (thePoint.y * y) + (thePoint.z * z);
    };
    Point3D opMul(float theS) {
        Point3D result;
        result.x = x * theS;
        result.y = y * theS;
        result.z = z * theS;
        return result;
    };
    float   length() {
        return sqrt( this.x*this.x + this.y*this.y + this.z*this.z );
    };
    void normalize() {
        float length =  length();
        this.x = this.x / length;
        this.y = this.y / length;
        this.z = this.z / length;
    };
    static Point3D cross(Point3D thePoint1, Point3D thePoint2) {
        Point3D result;
        result.x = thePoint1.y*thePoint2.z - thePoint1.z*thePoint2.y;
        result.y = thePoint1.z*thePoint2.x - thePoint1.x*thePoint2.z,
        result.z = thePoint1.x*thePoint2.y - thePoint1.y*thePoint2.x;
        return result;
    };
    float x;
    float y;
    float z;
};

