/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 2.1.2008
 *
 * The Renderer class renders the OpenGL Content
 *
 * The thing with the Engines standard: It is bad to change the stateswith every draw.
 * So DEngine is using a standard state and on a draw of something other than this state
 * it can be changed and should be changed back after drawing!
 *
 */
module renderer;

import derelict.opengl.gl;
import derelict.opengl.glu;
import singleton;
import task;
import config;

class Renderer : Task {
public:
    mixin Singleton;
    override void init() {
        myWidth  = cast(float)Config["renderer_width"];
        myHeight = cast(float)Config["renderer_height"];

        glViewport(0, 0, Config["renderer_width"], Config["renderer_height"]);

		glClearColor(0.5f, 0.5f, 0.8f, 1.0f);
		glShadeModel(GL_SMOOTH);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
		glDepthFunc(GL_LEQUAL);

        //Standard states
        glEnable(GL_DEPTH_TEST);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);

        join3D();
        DerelictGL.loadExtensions();
    };
    override int refresh() {
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();
        return 0;
    };
    override void uninit() {};
    void  join2D() {
        glMatrixMode (GL_PROJECTION);
        glLoadIdentity();
        glOrtho(0, myWidth, 0, myHeight,0,1);
        glMatrixMode (GL_MODELVIEW);
        glLoadIdentity();
    };
    void join3D() {
        glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45.0f, myWidth / myHeight, 1.0f, 1024.0f);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
    };
    float height()  { return myHeight; };
    float width()   { return myWidth; };

private:
    float myHeight;
    float myWidth;
};
