/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 17.1.2008
 *
 * ResourceManager: it loads and unloads Resources with thr rule to never own a Resource twice
 */
module resourcemanager;

import resource;
import singleton;

import std.stdio;

class ResourceManager {
    mixin Singleton;
    Resource need(ClassInfo ci, char[] theFilename) {
        foreach(res; myResources) { //search if resource already exists and then return it!
            if (res.classinfo == ci) {
                if (res.filename == theFilename) {  //returning existing resource
                    res.listener++;
                    //print();
                    return res;
                };
            };
        }; //When not returned, then that resource doesn t exist yet:
        Resource temp   = cast(Resource)ci.create();
        myResources     ~= temp;
        temp.load(theFilename);
        //print();
        return temp;    //return new created and to list added resource
    };
    void unNeed(Resource res) {
        for(int i=0; i<myResources.length; i++) {
            if (myResources[i] == res) {
                res.listener--;
                break;
            };
        };
        clear();    //call clear to check and unload non needed resources
    };
    void clear() { //check and unload non needed resources
        for(int i=0; i<myResources.length; i++) {
            if (myResources[i].listener == 0) {
                Resource temp = myResources[i];
                if (myResources[i] != myResources[length-1] ) myResources[i] = myResources[length-1];
                myResources.length = myResources.length - 1;
                temp.unload();
                delete temp;
            };
        };
        //print();
    };
    void print() {
        writefln("\nResourceManager: %d active Resources", myResources.length);
        foreach(res; myResources) writefln("\t%d %s %s", res.listener, res.classinfo.name, res.filename);
    }
private:
    Resource[]   myResources;
};
