/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 14.1.2008
 *
 * The SceneManager manages and renders all parts of the game elements.
 */
module scenemanager;

import camera;
import light;
import player;
import heightmap;
import texture2d;
import renderer;
import config;

import derelict.opengl.gl;

import std.stdio;

class SceneManager {
public:
    void init() {
        glClearColor(0.5f, 0.5f, 1.0f, 1.0f);
    };
    void addItem(Light theLight) {
        myLights ~= theLight;
    };
    void addItem(Player thePlayer) {
        myPlayers ~= thePlayer;
    };
    void addItem(Heightmap theHeightmap) {
        myHeightmaps ~= theHeightmap;
    };
    void addItem(Camera theCamera) {
        myCamera = theCamera;
    };
    void addItem(Texture2D theTexture) {
        myBackgroundTexture = theTexture;
    };


    void refresh() {

        Renderer.getInstance.join2D();

        glDisable(GL_DEPTH_TEST);
        glBindTexture(GL_TEXTURE_2D, myBackgroundTexture.texture);
        glBegin(GL_QUADS);
            glTexCoord2f(1,1); glVertex2f(0,0);
            glTexCoord2f(0,1); glVertex2f(Config["renderer_width"],0);
            glTexCoord2f(0,0); glVertex2f(Config["renderer_width"],Config["renderer_height"]);
            glTexCoord2f(1,0); glVertex2f(0,Config["renderer_height"]);
        glEnd();
        glEnable(GL_DEPTH_TEST);

        Renderer.getInstance.join3D();

        myCamera.refresh();

		foreach(player; myPlayers) {
			player.draw();
        };

        foreach(heightmap; myHeightmaps) {
            heightmap.draw(myCamera.position);
            //heightmap.draw();
        };
    };

    void uninit() {
        myLights.length     = 0;
        myPlayers.length    = 0;
        myHeightmaps.length = 0;
        myCamera            = null;
        myBackgroundTexture = null;
    };
private:
    Camera      myCamera;
    Light[]     myLights;
    Player[]    myPlayers;
    Heightmap[] myHeightmaps;
    Texture2D   myBackgroundTexture;
};
