/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 3.1.2008
 *
 * This is the Shader class.
 */
module shader;

import derelict.opengl.gl;
import derelict.opengl.extension.arb.vertex_program;
import derelict.opengl.extension.arb.fragment_program;

import std.stdio;

class Shader {

    void init() {

        char[]  stringVertex = "!!ARBvp1.0"
                        //# Eingangsattribute
                        "ATTRIB iPos = vertex.position;"
                        "ATTRIB iNormal = vertex.normal;"
                        //#Programmparameter
                        "PARAM mvinv[4] = { state.matrix.modelview.invtrans };"
                        "PARAM mvp[4] = { state.matrix.mvp };"
                        "PARAM lightDir = state.light[0].position;"
                        "PARAM halfDir = state.light[0].half;"
                        "PARAM specExp = state.material.shininess;"
                        "PARAM ambientCol = state.lightprod[0].ambient;"
                        "PARAM diffuseCol = state.lightprod[0].diffuse;"
                        "PARAM specularCol = state.lightprod[0].specular;"
                        //# Temporäre Variablen
                        "TEMP eyeNormal, temp, dots, lightcoefs;"
                        //# Ausgangsattribute
                        "OUTPUT oPos = result.position;"
                        "OUTPUT oColor = result.color;"
                        //# Vertex in Clipkoordinaten umwandeln
                        "DP4 oPos.x, mvp[0], iPos;"
                        "DP4 oPos.y, mvp[1], iPos;"
                        //"ADD mvp[2], mvp[2], 0.01f;"
                        "DP4 oPos.z, mvp[2], iPos;"
                        "DP4 oPos.w, mvp[3], iPos;"
                        //# Normale in eyespace transformieren
                        "DP3 eyeNormal.x, mvinv[0], iNormal;"
                        "DP3 eyeNormal.y, mvinv[1], iNormal;"
                        "DP3 eyeNormal.z, mvinv[2], iNormal;"
                        //# Diffuses und spekulatives Punktprodukt berechnen
                        //# und unter Nutzung von LIT die
                        //# Beleuchtungskoeffizienten berechnen
                        "DP3 dots.x, eyeNormal, lightDir;"
                        "DP3 dots.y, eyeNormal, halfDir;"
                        "MOV dots.w, specExp.x;"
                        "LIT lightcoefs, dots;"
                        //# Farbbestandteile sammeln und addieren
                        "MAD temp, lightcoefs.y, diffuseCol, ambientCol;"
                        "MAD oColor.xyz, lightcoefs.z, specularCol, temp;"
                        "MOV oColor.w, diffuseCol.w;"
                        "MUL oColor.xyz, vertex.color, temp;"
                        "END";

        char[]  stringFragment  =   "!!ARBfp1.0"
            "tex t0;"
            "MOV result.color, fragment.color;"


            "END";

        glGenProgramsARB(1, &vertexProgram);
        glGenProgramsARB(1, &fragmentProgram);

        glProgramStringARB(GL_VERTEX_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, stringVertex.length, stringVertex.ptr);
        glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, stringFragment.length, stringFragment.ptr);

        int err;
        glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &err);

        if(err != -1) {
            char* errorString;
            errorString = glGetString(GL_PROGRAM_ERROR_STRING_ARB);
            writefln("%d %s", err, *errorString);
        };


    };

    void enableVertexShader() {
        glEnable(GL_VERTEX_PROGRAM_ARB);
    };
    void disableVertexShader() {
        glDisable(GL_VERTEX_PROGRAM_ARB);
    };
    void enablePixelShader() {
        glEnable(GL_FRAGMENT_PROGRAM_ARB);
    };
    void disablePixelShader() {
        glDisable(GL_FRAGMENT_PROGRAM_ARB);
    };

    void uninit() {
    };

    uint    vertexProgram;
    uint    fragmentProgram;
};
