/**
 * Authors: Matthias Thurau, whosme.de
 * Date: 27.12.2007
 *
 * This is texturefont modul.
 */
module texturefont;

import derelict.opengl.gl;
import derelict.sdl.sdl;

import point;
import texture2d;
import image;
import resourcemanager;
import resource;

class TextureFont : Resource {
public:
    override void load(char[] theFilename) {
        myFilename      =   theFilename;
        myTexture       =   cast(Texture2D)ResourceManager.getInstance().need((Texture2D.classinfo), theFilename);
        Image myImage   =   cast(Image)ResourceManager.getInstance().need((Image.classinfo), theFilename);

        if (myImage.width >= 512) {
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        } else {
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        };

        myDimension         =   myImage.width;
        int letteramount    =   15;
        int maxlettersize   =   cast(int)myImage.width / letteramount;
        int letterstartxl;
        int letterstartxr  	=   myDimension;
        myLetters.length    =   (myDimension * myDimension) / 2;

        int currentLetter;
        uint  p;
        ubyte* r = new ubyte();
        ubyte* g = new ubyte();
        ubyte* b = new ubyte();
        ubyte* a = new ubyte();
        for (int y=0; y < letteramount; y++)
        {
            for (int x=0; x < letteramount; x++)
            {
                for (int x2=x*maxlettersize; x2 < (x+1)*maxlettersize; x2++)
                {
                    for (int y2=y*maxlettersize; y2 < (y+1)*maxlettersize; y2++)
                    {
                        p = get_pixel(myImage.surface, x2, y2);
                        if (myImage.iformat == GL_RGBA)
                        {
                            SDL_GetRGBA(p, myImage.surface.format, r, g, b, a);
                            if (*a != 0)
                            {
                                letterstartxl   =   x2;
                                y2              =   myImage.height;
                                x2              =   myImage.width;
                            };
                        }
                        else
                        {
                            if (p!=0)
                            {
                                letterstartxl   =   x2;
                                y2              =   myImage.height;
                                x2              =   myImage.width;
                            };
                        };
                    };
                };
                for (int x2=(x+1)*maxlettersize; x2 > x*maxlettersize; x2--)
                {
                    for (int y2=y*maxlettersize; y2 < (y+1)*maxlettersize; y2++)
                    {
                        p = get_pixel(myImage.surface, x2, y2);
                        if (myImage.iformat == GL_RGBA)
                        {
                            SDL_GetRGBA(p, myImage.surface.format, r, g, b, a);
                            if (*a != 0)
                            {
                                letterstartxr   =   x2+1;
                                y2              =   myImage.height;
                                x2              =   0;
                            };
                        }
                        else
                        {
                            if (p!=0)
                            {
                                letterstartxr   =   x2+1;
                                y2              =   myImage.height;
                                x2              =   0;
                            };
                        };
                    };
                };
                if ((((x*letteramount)+y)%2) == 0) {
                    myLetters[currentLetter].uv[0].x = cast(float) letterstartxl / myDimension;
                    myLetters[currentLetter].uv[0].y = cast(float) (y  *maxlettersize)     / myDimension;
                    myLetters[currentLetter].uv[1].x = cast(float) letterstartxr / myDimension;
                    myLetters[currentLetter].uv[1].y = cast(float) (y  *maxlettersize)     / myDimension;
                    myLetters[currentLetter].uv[2].x = cast(float) letterstartxr / myDimension;
                    myLetters[currentLetter].uv[2].y = cast(float) ((y+1)*maxlettersize)   / myDimension;
                    myLetters[currentLetter].uv[3].x = cast(float) letterstartxl / myDimension;
                    myLetters[currentLetter].uv[3].y = cast(float) ((y+1)*maxlettersize)   / myDimension;
                } else {
                    myLetters[currentLetter].uv[4].x = cast(float) letterstartxl / myDimension;
                    myLetters[currentLetter].uv[4].y = cast(float) (y  *maxlettersize)     / myDimension;
                    myLetters[currentLetter].uv[5].x = cast(float) letterstartxr / myDimension;
                    myLetters[currentLetter].uv[5].y = cast(float) (y  *maxlettersize)     / myDimension;
                    myLetters[currentLetter].uv[6].x = cast(float) letterstartxr / myDimension;
                    myLetters[currentLetter].uv[6].y = cast(float) ((y+1)*maxlettersize)   / myDimension;
                    myLetters[currentLetter].uv[7].x = cast(float) letterstartxl / myDimension;
                    myLetters[currentLetter].uv[7].y = cast(float) ((y+1)*maxlettersize)   / myDimension;
                    currentLetter++;
                };
            };
        };
        delete r;
        delete g;
        delete b;
        delete a;
        ResourceManager.getInstance.unNeed(myImage);
    };

    override void unload() {
        ResourceManager.getInstance.unNeed(myTexture);
    };
    uint get_pixel(SDL_Surface *surface, int x, int y)
    {
        int bpp = surface.format.BytesPerPixel;
        Uint8 *p = cast(ubyte *)surface.pixels + y * surface.pitch + x * bpp;
        switch(bpp) {
        case 1:
            return *p;
        case 2:
            return *cast(ushort *)p;
        case 3:
            if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
                return p[0] << 16 | p[1] << 8 | p[2];
            else
                return p[0] | p[1] << 8 | p[2] << 16;
        case 4:
            return *cast(uint *)p;
        default:
            break;
        };
        return 0;
    };
    Letter[]    letters()   { return myLetters; };
    Texture2D   texture()   { return myTexture; };
    int         dimension() { return myDimension; };

private:
    Texture2D   myTexture;
    Letter[]    myLetters;
    int         myDimension;
};
struct Letter {
    int			sdl_key;
    int 		unicode;
    Point2D[8]	uv;     //first 4: normal; last 4: bold;
};
